@@ -2,7 +2,7 @@ use strict;
 use warnings;
 
 package Net::OpenNebula::RPC;
-$Net::OpenNebula::RPC::VERSION = '0.305.0';
+$Net::OpenNebula::RPC::VERSION = '0.306.0';
 use Data::Dumper;
 
 use constant ONERPC => 'rpc';
@@ -17,11 +17,11 @@ sub new {
    my $proto = ref($that) || $that;
    my $self = { @_ };
 
-   $self->{ONERPC} = $proto->ONERPC; 
+   $self->{ONERPC} = $proto->ONERPC;
 
    $self->{cache}->{add} = 0;
    $self->{cache}->{remove} = 0;
-   
+
    bless($self, $proto);
 
    return $self;
@@ -41,19 +41,19 @@ sub _onerpc {
         $self->{rpc}->add_cache_method($onemethod);
         $self->{cache}->{add} = 0;
     }
-		
+
     return $self->{rpc}->_rpc($onemethod, @args);
 }
 
 sub _onerpc_id {
     my ($self, $method) = @_;
-    
+
     $self->has_id("_onerpc_id") || return;
 
     return $self->_onerpc($method,
                             [ int => $self->id ],
                           );
-    
+
 };
 
 sub _onerpc_simple {
@@ -71,13 +71,13 @@ sub _onerpc_simple {
 # return info call
 # opts
 #   clearcache: if set to 1, clears the cache and queries again
-#   id: get info for other id (if missing, use $self->id) 
+#   id: get info for other id (if missing, use $self->id)
 sub _get_info {
     my ($self, %option) = @_;
 
     my $id;
     if (exists $option{id}) {
-        $id = $option{id} ;  
+        $id = $option{id} ;
     } else {
         $self->has_id("_get_info") || return;
         $id = $self->id;
@@ -88,21 +88,21 @@ sub _get_info {
     }
 }
 
-# Similar to _get_info, but will try with with clearcache if C<entry> can't be 
-# found in extended_data and it returns the entry in extended_data. 
+# Similar to _get_info, but will try with with clearcache if C<entry> can't be
+# found in extended_data and it returns the entry in extended_data.
 sub _get_info_extended {
     my ($self, $entry) = @_;
     $self->_get_info();
-    
+
     if(! exists $self->{extended_data}->{$entry}) {
         $self->_get_info(clearcache => 1);
     }
 
     if(exists $self->{extended_data}->{$entry}) {
-        $self->debug(2, "Entry $entry present in extended_data");    
+        $self->debug(2, "Entry $entry present in extended_data");
         return $self->{extended_data}->{$entry};
     } else {
-        $self->debug(2, "Entry $entry still not present in extended_data");    
+        $self->debug(2, "Entry $entry still not present in extended_data");
         return []; # empty array ref
     }
 }
@@ -138,11 +138,11 @@ sub _allocate {
     if (! defined($id)) {
         my $args_txt = $self->{rpc}->_rpc_args_to_txt(@args);
         $self->error("$self->{ONERPC}: _allocate failed, no id returned (arguments $args_txt).");
-        return;        
+        return;
     }
 
     $self->debug(1, "$self->{ONERPC} allocate returned id $id");
-    
+
     my $data = $self->_get_info(id => $id);
     if (defined($data)) {
         $self->{data} = $data;
@@ -182,8 +182,99 @@ sub update {
                           );
 }
 
+# Chown
+# options
+#    uid : uid to use
+#    gid : gid to use
+#    one : Net::OpenNebula instance to use for user/groupname lookup
+#          (i.e. when uid/gid is not a integer)
+#          It wil use $one->get_users and/or $one->get_groups methods.
 
-# When C<nameregex> is defined, only instances with name matching 
+sub _lookup
+{
+    my ($self, $type, $id, $one) = @_;
+
+    $id = -1 if ! defined($id);
+
+    my $err;
+    if ($id !~ m/^-?\d+$/) {
+        my $msg = "Non-integer ${type}name $id";
+        if ($one) {
+            my $method = "get_${type}s";
+            $self->debug(1, "$msg, looking up id using $method");
+            my @found = $one->$method($id);
+            if (@found) {
+                if (scalar @found > 1) {
+                    $self->warn("$msg and more than one $type found: ".join(", ", @found));
+                }
+                $id = $found[0]->id();
+                $self->debug(1, "$msg resolved to id $id");
+            } else {
+                $err = "$msg and no matching ${type}s found.";
+            }
+        } else {
+            $err = "$msg and no ONE instance provided";
+        }
+    }
+
+    if ($err) {
+        $self->error($err);
+        return;
+    } else {
+        return $id;
+    }
+}
+
+
+sub chown
+{
+    my ($self, %opts) = @_;
+
+    $self->has_id("chown") || return;
+
+    my $uid = $self->_lookup('user', $opts{uid}, $opts{one});
+    return if ! defined($uid);
+
+    my $gid = $self->_lookup('group', $opts{gid}, $opts{one});
+    return if ! defined($gid);
+
+    return $self->_onerpc("chown",
+                          [ int => $self->id ],
+                          [ int => $uid ],
+                          [ int => $gid ]
+                          );
+}
+
+
+# Mode can be
+#   an integer, typical in octal mode 0664, but can be decimal or whatever
+sub chmod
+{
+    my ($self, $mode) = @_;
+
+    $self->has_id("chmod") || return;
+
+    my @bits;
+    if ($mode =~ m/^\d+/) {
+        my $bin = sprintf("%b", $mode & oct("777"));
+        @bits = split('', $bin);
+    } else {
+        $self->error("chown cannot handle mode $mode");
+        return;
+    };
+
+    if((scalar @bits) == 9) {
+        my @allargs = qw(chmod);
+        push(@allargs, [ int => $self->id ], map {[ int => $_ ]} @bits);
+        return $self->_onerpc(@allargs);
+    } else {
+        $self->error("9 permissions bits required, got ".join(",", @bits));
+        return;
+    }
+}
+
+
+# When C<nameregex> is defined, only instances with name matching
 # the regular expression are returned (if any).
 # C<nameregex> is a compiled regular expression (e.g. qr{^somename$}).
 sub _get_instances {
@@ -196,30 +287,30 @@ sub _get_instances {
     my @ret = ();
 
     my $reply = $self->{rpc}->_rpc("one.$pool.info", @args);
-   
+
     for my $data (@{ $reply->{$key} }) {
-        my $inst = $self->new(rpc => $self->{rpc}, data => $data); 
-        if (! defined($nameregex) || ($inst->name && $inst->name =~ $nameregex) ) { 
+        my $inst = $self->new(rpc => $self->{rpc}, data => $data);
+        if (! defined($nameregex) || ($inst->name && $inst->name =~ $nameregex) ) {
             push(@ret, $inst);
-        }   
+        }
     }
-    
+
     return @ret;
 }
 
-# Given state, wait until the state is reached. 
-# Between each check, sleep number of seconds; 
-# and there's a maximum number of iterations to try. 
-# Return 1 if the state is reached, 0 otherwise. 
+# Given state, wait until the state is reached.
+# Between each check, sleep number of seconds;
+# and there's a maximum number of iterations to try.
+# Return 1 if the state is reached, 0 otherwise.
 # state: the state (in text) to wait for
 # opts:
 #    sleep: sleep per interval
 #    max_iter: maximum iterations (if 0, no sleep)
 sub wait_for_state {
     my ($self, $state, %opts) = @_;
-    
+
     my $sleep = 5; # in seconds
-    my $max_iter = 200; # approx 15 minutes with default sleep 
+    my $max_iter = 200; # approx 15 minutes with default sleep
     $sleep = $opts{sleep} if defined($opts{sleep});
     $max_iter = $opts{max_iter} if defined($opts{max_iter});
 
@@ -230,9 +321,9 @@ sub wait_for_state {
         $currentstate = $state eq $self->state;
         $ind +=1;
     }
-    
+
     return $currentstate;
-   
+
 }
 
 # add logging shortcuts
